@@ -5,19 +5,19 @@
 package PDL::CCS::Compat;
 use PDL;
 use PDL::VectorValued;
-use PDL::CCS::Config qw(ccs_indx);
+use PDL::CCS::Version;
 use PDL::CCS::Functions;
 use PDL::CCS::Utils;
 use PDL::CCS::Ufunc;
 use PDL::CCS::Ops;
 use strict;
 
-our $VERSION = '1.24.0'; ##-- update with perl-reversion from Perl::Version module
+our $VERSION = $PDL::CCS::VERSION;
 our @ISA = ('PDL::Exporter');
 our @ccs_binops = (qw(plus minus mult divide modulo power),
-                   qw(gt ge lt le eq ne spaceship),
-                   qw(and2 or2 xor shiftleft shiftright),
-                  );
+		   qw(gt ge lt le eq ne spaceship),
+		   qw(and2 or2 xor shiftleft shiftright),
+		  );
 our @EXPORT_OK =
   (
    ##
@@ -150,9 +150,9 @@ PDL::CCS::Compat - Backwards-compatibility module for PDL::CCS
 
 =for sig
 
-  Signature: (indx awhich(2,Nnz); avals(Nnz);
-              indx $N; indx $M;
-              indx [o]ptr(N); indx [o]rowids(Nnz); [o]nzvals(Nnz))
+  Signature: (int awhich(2,Nnz); avals(Nnz);
+              int $N; int $M;
+              int [o]ptr(N); int [o]rowids(Nnz); [o]nzvals(Nnz))
 
 Generic wrapper for backwards-compatible ccsencode() variants.
 
@@ -189,7 +189,7 @@ sub PDL::ccs_encode_compat {
 
 =for sig
 
-  Signature: (a(N,M); indx [o]ptr(N); indx [o]rowids(Nnz); [o]nzvals(Nnz))
+  Signature: (a(N,M); int [o]ptr(N); int [o]rowids(Nnz); [o]nzvals(Nnz))
 
 Encodes matrix $a() in compressed column format, interpreting zeroes
 as "missing" values.
@@ -205,8 +205,7 @@ Allocates output vectors if required.
   = \&ccsencode_nz;
 sub ccsencode_nz {
   #my ($a,$ptr,$rowids,$nzvals) = @_;
-  my $a  = shift;
-  $a     = $a->clump(1+$a->ndims-2); ##-- clump(-2) broken in PDL-2.0.14
+  my $a  = shift->clump(-2);
   my $aw = $a->whichND;
   return ccs_encode_compat($aw, $a->indexND($aw), $a->dims, @_);
 }
@@ -221,7 +220,7 @@ sub ccsencode_nz {
 
 =for sig
 
-  Signature: (a(N,M); eps(); indx [o]ptr(N); indx [o]rowids(Nnz); [o]nzvals(Nnz))
+  Signature: (a(N,M); eps(); int [o]ptr(N); int [o]rowids(Nnz); [o]nzvals(Nnz))
 
 Encodes matrix $a() in CCS format interpreting approximate zeroes as "missing" values.
 This function is just like ccsencode_nz(), but uses the tolerance parameter
@@ -238,9 +237,8 @@ Allocates output vectors if required.
   = \&ccsencode_naz;
 sub ccsencode_naz {
   #my ($a,$eps,$ptr,$rowids,$nzvals) = @_;
-  my $a   = shift;
+  my $a   = shift->clump(-2);
   my $eps = shift;
-  $a      = $a->clump(1+$a->ndims-2); ##-- clump(-2) is broken in PDL-2.014
   my $aw  = $a->approx(0,$eps)->inplace->not->whichND;          ##-- FIXME: optimize
   return ccs_encode_compat($aw, $a->indexND($aw), $a->dims, @_);
 }
@@ -255,7 +253,7 @@ sub ccsencode_naz {
 
 =for sig
 
-  Signature: (a(N,M); indx [o]ptr(N); indx [o]rowids(Nnz); [o]nzvals(Nnz))
+  Signature: (a(N,M); int [o]ptr(N); int [o]rowids(Nnz); [o]nzvals(Nnz))
 
 Encodes matrix $a() in CCS format interpreting BAD values
 as "missing".  Requires bad-value support built into PDL.
@@ -271,8 +269,7 @@ Allocates output vectors if required.
   = \&ccsencode_g;
 sub ccsencode_g {
   #my ($a,$ptr,$rowids,$nzvals) = @_;
-  my $a     = shift;
-  $a        = $a->clump(1+$a->ndims-2);  ##-- clump(-2) is broken in PDL-v2.014
+  my $a     = shift->clump(-2);
   my $amask = zeroes(byte,$a->dims);
   $a->isgood($amask);
   my $aw    = $amask->whichND;
@@ -287,7 +284,7 @@ sub ccsencode_g {
 
 =for sig
 
-  Signature: (indx ix(Nnz); nzvals(Nnz); indx $N; int [o]ptr(N); indx [o]rowids(Nnz); [o]nzvals_enc(Nnz))
+  Signature: (int ix(Nnz); nzvals(Nnz); int $N; int [o]ptr(N); int [o]rowids(Nnz); [o]nzvals_enc(Nnz))
 
 General-purpose CCS encoding method for flat indices.
 Encodes values $nzvals() from flat-index locations $ix() into a CCS matrix ($ptr(), $rowids(), $nzvals_enc()).
@@ -321,13 +318,13 @@ sub ccsencode_i {
 =for sig
 
   Signature: (
-              indx    xvals(Nnz)     ;
-              indx    yvals(Nnz)     ;
-                      nzvals(Nnz)    ;
-              indx    $N             ; ##-- optional
-              indx [o]ptr(N)         ;
-              indx [o]rowids(Nnz)    ;
-                   [o]nzvals_enc(Nnz);
+              int  xvals(Nnz)       ;
+              int  yvals(Nnz)       ;
+                  nzvals(Nnz)       ;
+              int       $N          ; ##-- optional
+              int [o]ptr(N)         ;
+              int [o]rowids(Nnz)    ;
+                  [o]nzvals_enc(Nnz);
              )
 
 General-purpose encoding method.
@@ -370,10 +367,10 @@ sub ccsencode_i2d {
 =for sig
 
   Signature: (
-              indx   ptr    (N)  ;
-              indx   rowids (Nnz);
+              int    ptr    (N)  ;
+              int    rowids (Nnz);
                      nzvals (Nnz);
-              indx   xvals  (I)  ; # default=sequence($N)
+              int    xvals  (I)  ; # default=sequence($N)
                      missing()   ; # default=0
                      M      ()   ; # default=rowids->max+1
                   [o]cols   (I,M); # default=new
@@ -392,16 +389,16 @@ following two calls are equivalent (modulo data flow):
 
 
 *PDL::_ccsdecodecols = \&_ccsdecodecols;
-#Pars => 'indx ptr(N); indx rowids(Nnz); nzvals(Nnz); indx col_ix(I); missing(); [o]cols(I,M);',
+#Pars => 'int ptr(N); int rowids(Nnz); nzvals(Nnz); int col_ix(I); missing(); [o]cols(I,M);',
 sub _ccsdecodecols {
   ccsdecodecols(@_[0,1,2], $_[3],$_[4], undef, $_[5]);
 }
 *PDL::ccsdecodecols = \&ccsdecodecols;
 sub ccsdecodecols {
   my ($ptr,$rowids,$nzvals, $coli,$missing,$M, $cols) = @_;
-  $coli    = sequence(ccs_indx,$ptr->dim(0)) if (!defined($coli));
-  $coli    = pdl(ccs_indx,$coli)             if (!ref($coli));
-  my $ptr1 = zeroes(ccs_indx,$ptr->nelem+1);
+  $coli    = sequence(long,$ptr->dim(0)) if (!defined($coli));
+  $coli    = pdl(long,$coli)             if (!ref($coli));
+  my $ptr1 = zeroes(long,$ptr->nelem+1);
   $ptr1->slice("0:-2") .= $ptr;
   $ptr1->set(-1 => $nzvals->nelem);
   $M = $rowids->max+1 if (!defined($M));
@@ -425,7 +422,7 @@ sub ccsdecodecols {
 
 =for sig
 
-  Signature: (indx ptr(N); indx rowids(Nnz); nzvals(Nnz); $M; [o]dense(N,M))
+  Signature: (int ptr(N); int rowids(Nnz); nzvals(Nnz); $M; [o]dense(N,M))
 
 Decodes compressed column format vectors $ptr(), $rowids(), and $nzvals()
 into dense output matrix $a().
@@ -438,7 +435,7 @@ In such cases, you might prefer to call ccsdecodecols() directly.
 
 =cut
 
-*PDL::ccsdecodefull = \&ccsdecodefull; ##-- (indx ptr(N); indx rowids(Nnz); nzvals(Nnz); [o]dense(N,M))
+*PDL::ccsdecodefull = \&ccsdecodefull; ##-- (int ptr(N); int rowids(Nnz); nzvals(Nnz); [o]dense(N,M))
 sub ccsdecodefull { ccsdecodecols(@_[0,1,2], undef,0,undef, @_[3..$#_]); }
 
 *PDL::ccsdecode = \&ccsdecode;
@@ -466,7 +463,7 @@ sub ccsdecode {
 
 =for sig
 
-  Signature: (indx ptr(N); indx rowids(Nnz); nzvals(Nnz); $M; [o]dense(N,M))
+  Signature: (int ptr(N); int rowids(Nnz); nzvals(Nnz); $M; [o]dense(N,M))
 
 Convenience method.
 Like ccsdecode() but sets "missing" values to BAD.
@@ -513,7 +510,7 @@ sub ccsdecode_g {
 
 =for sig
 
-  Signature: (indx ptr(N); indx rowids(Nnz); indx ind(2,I); indx missing(); indx [o]nzix(I))
+  Signature: (int ptr(N); int rowids(Nnz); int ind(2,I); int missing(); int [o]nzix(I))
 
 =head2 ccsiNDtonzi
 
@@ -532,7 +529,6 @@ sub ccsiNDtonzi {
   $nzix              = $ind->vsearchvec($ccswnd);
   my $nzix_mask      = ($ind==$ccswnd->dice_axis(1,$nzix))->andover;
   $nzix_mask->inplace->not;
-  #(my $tmp = $nzix->where($nzix_mask)) .= $missing; ##-- fix "Can't modify non-lvalue subroutine call" in 5.15.x (perl bug #107366)
   $nzix->where($nzix_mask) .= $missing;
   return $nzix;
 }
@@ -546,7 +542,7 @@ sub ccsiNDtonzi {
 
 =for sig
 
-  Signaure: (indx ptr(N); indx rowids(Nnz); indx col_ix(I); indx row_ix(I); indx missing(); indx [o]nzix(I))
+  Signaure: (int ptr(N); int rowids(Nnz); int col_ix(I); int row_ix(I); int missing(); int [o]nzix(I))
 
 Convert 2d index values $col_ix() and $row_ix() appropriate for a dense matrix (N,M)
 into indices $nzix() appropriate for the $rowids() and/or $nzvals() components
@@ -569,7 +565,7 @@ sub ccsi2dtonzi {
 
 =for sig
 
-  Signature: (indx ptr(N); indx rowids(Nnz); indx ix(I); indx missing(); indx [o]nzix(I))
+  Signature: (int ptr(N); int rowids(Nnz); int ix(I); int missing(); int [o]nzix(I))
 
 =head2 ccsitonzi
 
@@ -603,7 +599,7 @@ sub ccsitonzi {
 
 =for sig
 
-  Signature: (indx ptr(N); indx rowids(Nnz); nzvals(Nnz); indx [o]which_cols(Nnz); indx [o]which_rows(Nnz)',
+  Signature: (int ptr(N); int rowids(Nnz); nzvals(Nnz); int [o]which_cols(Nnz); int [o]which_rows(Nnz)',
 
 In scalar context, returns concatenation of $which_cols() and $which_rows(),
 similar to the builtin whichND().  Note however that ccswhichND() may return
@@ -619,10 +615,10 @@ sub ccswhichND {
   my ($ptr,$rowids,$nzvals, $which_cols,$which_rows) = @_;
   my ($ptrnzi);
   ($which_cols,$ptrnzi) = ccs_decode_pointer($ptr->append($rowids->nelem),
-                                             sequence(ccs_indx, $ptr->nelem),
-                                             $which_cols
-                                            );
-  $which_rows  = zeroes(ccs_indx, $rowids->nelem) if (!defined($which_rows));
+					     sequence(long, $ptr->nelem),
+					     $which_cols
+					    );
+  $which_rows  = zeroes(long, $rowids->nelem) if (!defined($which_rows));
   $which_rows .= $rowids->index($ptrnzi);
   return wantarray ? ($which_cols,$which_rows) : $which_cols->cat($which_rows)->xchg(0,1);
 }
@@ -635,7 +631,7 @@ sub ccswhichND {
 
 =for sig
 
-  Signature: (indx ptr(N); indx rowids(Nnz); nzvals(Nnz); indx [o]which(Nnz); indx [t]wcols(Nnz)',
+  Signature: (int ptr(N); int rowids(Nnz); nzvals(Nnz); int [o]which(Nnz); int [t]wcols(Nnz)',
 
 Convenience method.
 Calls ccswhichfull(), and scales the output PDLs to correspond to a flat enumeration.
@@ -648,8 +644,8 @@ Use the qsort() method if you need sorted output.
 sub ccswhich {
   my ($ptr,$rowids,$nzvals, $which, $wcols) = @_;
   my $nnz = $rowids->dim(0);
-  $which = zeroes(ccs_indx,$nnz) if (!defined($which));
-  $wcols = zeroes(ccs_indx,$nnz) if (!defined($wcols));
+  $which = zeroes(long,$nnz) if (!defined($which));
+  $wcols = zeroes(long,$nnz) if (!defined($wcols));
   ccswhichfull($ptr,$rowids,$nzvals, $wcols,$which);
   $which *= $ptr->dim(0);
   $which += $wcols;
@@ -665,7 +661,7 @@ sub ccswhich {
 
 =for sig
 
-  Signature: (indx ptr(N); indx rowids(Nnz); nzvals(Nnz); indx [o]ptrT(M); indx [o]rowidsT(Nnz); [o]nzvalsT(Nnz)',
+  Signature: (int ptr(N); int rowids(Nnz); nzvals(Nnz); int [o]ptrT(M); int [o]rowidsT(Nnz); [o]nzvalsT(Nnz)',
 
 Transpose a compressed matrix.
 
@@ -700,7 +696,7 @@ sub ccstranspose {
 
 =for sig
 
-  Signature: (indx ptr(N); indx rowids(Nnz); nzvals(Nnz); indx xvals(I); indx yvals(I); missing(); [o]ixvals(I))
+  Signature: (int ptr(N); int rowids(Nnz); nzvals(Nnz); int xvals(I); int yvals(I); missing(); [o]ixvals(I))
 
 Lookup values in a CCS-encoded PDL by 2d source index (no dataflow).
 Pretty much like ccsi2dtonzi(), but returns values instead of indices.
@@ -722,12 +718,8 @@ sub ccsget2d {
   my $nzi        = ccsi2dtonzi($ptr,$rowids, $xi,$yi, -1);
   my $nzi_isgood = ($nzi != -1);
   $ixnzvals      = zeroes($nzvals->type, $xi->nelem) if (!defined($ixnzvals));
-  if (!all($nzi_isgood)) {
-    my $tmp;
-    ($tmp=$ixnzvals->where( $nzi_isgood)) .= $nzvals->index($nzi->where($nzi_isgood));
-    ($tmp=$ixnzvals->where(!$nzi_isgood)) .= $missing;
-    $ixnzvals->badflag(1) if (PDL->topdl($missing)->badflag);
-  }
+  $ixnzvals->where( $nzi_isgood) .= $nzvals->index($nzi->where($nzi_isgood));
+  $ixnzvals->where(!$nzi_isgood) .= $missing;
   return $ixnzvals;
 }
 
@@ -741,7 +733,7 @@ sub ccsget2d {
 
 =for sig
 
-  Signature: (indx ptr(N); indx rowids(Nnz); nzvals(Nnz); indx ix(I); missing(); [o]ixvals(I))
+  Signature: (int ptr(N); int rowids(Nnz); nzvals(Nnz); int ix(I); missing(); [o]ixvals(I))
 
 Lookup values in a CCS-encoded PDL by flat source index (no dataflow).
 Pretty much like ccsitonzi(), but returns values instead of indices.
@@ -763,12 +755,8 @@ sub ccsget {
   my $nzi        = ccsitonzi($ptr,$rowids, $ix,-1);
   my $nzi_isgood = ($nzi != -1);
   $ixnzvals      = zeroes($nzvals->type, $ix->nelem) if (!defined($ixnzvals));
-  if (!all($nzi_isgood)) {
-    my $tmp;
-    ($tmp=$ixnzvals->where( $nzi_isgood)) .= $nzvals->index($nzi->where($nzi_isgood));
-    ($tmp=$ixnzvals->where(!$nzi_isgood)) .= $missing;
-    $ixnzvals->badflag(1) if (PDL->topdl($missing)->badflag);
-  }
+  $ixnzvals->where( $nzi_isgood) .= $nzvals->index($nzi->where($nzi_isgood));
+  $ixnzvals->where(!$nzi_isgood) .= $missing;
   return $ixnzvals;
 }
 
@@ -791,7 +779,7 @@ sub ccsget {
 
 =for sig
 
-  Signature: (indx ptr(N); indx rowids(Nnz); nzvals_in(Nnz);  colvec(M);  [o]nzvals_out(Nnz))
+  Signature: (int ptr(N); int rowids(Nnz); nzvals_in(Nnz);  colvec(M);  [o]nzvals_out(Nnz))
 
 Column-vector operation ${OP} on CCS-encoded PDL.
 
@@ -832,8 +820,7 @@ sub ccs_binop_compat_cv {
   return sub { $ccsop->(@_[1,2,3,4]) };
 }
 foreach my $op (@ccs_binops) {
-  no strict 'refs';
-  *{"ccs${op}_cv"} = *{"PDL::ccs${op}_cv"} = ccs_binop_compat_cv(\&{"PDL::ccs_${op}_vector_mia"});
+  eval "*ccs${op}_cv = *PDL::ccs${op}_cv = ccs_binop_compat_cv(\\&PDL::ccs_${op}_vector_mia);";
 }
 *ccsadd_cv = *PDL::ccsadd_cv = \&ccsplus_cv;
 *ccsdiff_cv = *PDL::ccsdiff_cv = \&ccsminus_cv;
@@ -848,7 +835,7 @@ foreach my $op (@ccs_binops) {
 
 =for sig
 
-  Signature: (indx ptr(N); indx rowids(Nnz); nzvals_in(Nnz);  rowvec(N);  [o]nzvals_out(Nnz))
+  Signature: (int ptr(N); int rowids(Nnz); nzvals_in(Nnz);  rowvec(N);  [o]nzvals_out(Nnz))
 
 Row-vector operation ${OP} on CCS-encoded PDL.
 Should do something like the following (without decoding the CCS matrix):
@@ -870,8 +857,7 @@ sub ccs_binop_compat_rv {
   };
 }
 foreach my $op (@ccs_binops) {
-  no strict 'refs';
-  *{"ccs${op}_rv"} = *{"PDL::ccs${op}_rv"} = ccs_binop_compat_rv(\&{"PDL::ccs_${op}_vector_mia"});
+  eval "*ccs${op}_rv = *PDL::ccs${op}_rv = ccs_binop_compat_rv(\\&PDL::ccs_${op}_vector_mia);";
 }
 *ccsadd_rv = *PDL::ccsadd_rv = \&ccsplus_rv;
 *ccsdiff_rv = *PDL::ccsdiff_rv = \&ccsminus_rv;
@@ -925,13 +911,13 @@ sub ccs_ufunc_compat_t {
 =head2 Compressed Column Format Example
 
  $a = pdl([
-           [10, 0, 0, 0,-2,  0],
-           [3,  9, 0, 0, 0,  3],
-           [0,  7, 8, 7, 0,  0],
-           [3,  0, 8, 7, 5,  0],
-           [0,  8, 0, 9, 9, 13],
-           [0,  4, 0, 0, 2, -1]
-          ]);
+	   [10, 0, 0, 0,-2,  0],
+	   [3,  9, 0, 0, 0,  3],
+	   [0,  7, 8, 7, 0,  0],
+	   [3,  0, 8, 7, 5,  0],
+	   [0,  8, 0, 9, 9, 13],
+	   [0,  4, 0, 0, 2, -1]
+	  ]);
 
  ($ptr,$rowids,$nzvals) = ccsencode($a);
 
@@ -1012,7 +998,7 @@ Bryan Jurish E<lt>moocow@cpan.orgE<gt>
 
 =head2 Copyright Policy
 
-Copyright (C) 2005-2024, Bryan Jurish. All rights reserved.
+Copyright (C) 2005-2007, Bryan Jurish. All rights reserved.
 
 This package is free software, and entirely without warranty.
 You may redistribute it and/or modify it under the same terms
