@@ -2,7 +2,7 @@
 ## Author: Bryan Jurish
 ## Description: set version for PDL::CCS
 
-package PDL::CCS::Version;
-our $VERSION = '1.24.1'; ##-- update with perl-reversion from Perl::Version module
+package PDL::CCS;
+our $VERSION=1.14;
 
 1; ##-- make perl happy
